#include <windows.h>
#include <stdio.h>
//#include <stdlib.h>
//#include <string.h>

void ReportError(char *upname) {
 printf ("failure: %s (%u)\n",upname,GetLastError());
 FormatMessage(FORMAT_MESSAGE_ALLOCATE_BUFFER|FORMAT_MESSAGE_FROM_SYSTEM,
   NULL,
   GetLastError(),
   MAKELANGID(LANG_NEUTRAL,SUBLANG_DEFAULT),
   (LPTSTR) &upname,
   0,
   NULL);
 CharToOem(upname,upname);
 printf("%s\n",upname);
 LocalFree(upname);
}

BOOL InstallDriver(SC_HANDLE SchSCManager, LPCTSTR DriverName,
  LPCTSTR ServiceExe) {
 SC_HANDLE  schService;
 DWORD      err;
    //
    // NOTE: This creates an entry for a standalone driver. If this
    //       is modified for use with a driver that requires a Tag,
    //       Group, and/or Dependencies, it may be necessary to
    //       query the registry for existing driver information
    //       (in order to determine a unique Tag, etc.).
    //
 schService = CreateService (SchSCManager,          // SCManager database
                             DriverName,           // name of service
                             "DongleKnacker",           // name to display
                             SERVICE_START|SERVICE_STOP,    // desired access
                             SERVICE_KERNEL_DRIVER, // service type
                             SERVICE_DEMAND_START,  // start type
                             SERVICE_ERROR_NORMAL,  // error control type
                             ServiceExe,            // service's binary
                             NULL,                  // no load ordering group
                             NULL,                  // no tag identifier
                             NULL,                  // no dependencies
                             NULL,                  // LocalSystem account
                             NULL);                 // no password
 if (schService == NULL) {
  ReportError("CreateService");
  return FALSE;
 }else{
  printf ("CreateService SUCCESS\n");
 }

 CloseServiceHandle (schService);
 return TRUE;
}

BOOL RemoveDriver(SC_HANDLE SchSCManager, LPCTSTR DriverName) {
 SC_HANDLE  schService;
 BOOL       ret;

 schService = OpenService (SchSCManager,DriverName,DELETE);
 if (!schService) {
  ReportError("OpenService");
  return FALSE;
 }
 ret = DeleteService (schService);
 if (ret) {
  printf ("DeleteService SUCCESS\n");
 }else{
  ReportError("DeleteService");
 }
 CloseServiceHandle (schService);
 return ret;
}

BOOL StartDriver(SC_HANDLE SchSCManager, LPCTSTR DriverName) {
 SC_HANDLE  schService;
 BOOL       ret;

 schService = OpenService (SchSCManager,DriverName,SERVICE_START);
 if (!schService){
  ReportError("OpenService");
  return FALSE;
 }
 ret = StartService (schService,    // service identifier
		     0,             // number of arguments
		     NULL);         // pointer to arguments
 if (ret) {
  printf ("StartService SUCCESS\n");
 }else{
  ReportError("StartService");
 }
 CloseServiceHandle (schService);
 return ret;
}

BOOL StopDriver(SC_HANDLE SchSCManager, LPCTSTR DriverName) {
 SC_HANDLE       schService;
 BOOL            ret;
 SERVICE_STATUS  serviceStatus;

 schService = OpenService (SchSCManager,DriverName,SERVICE_START|SERVICE_STOP);
 if (!schService) {
  ReportError("OpenService");
  return FALSE;
 }
 ret = ControlService (schService,SERVICE_CONTROL_STOP,&serviceStatus);
 if (ret) {
  printf ("ControlService SUCCESS\n");
 }else{
  ReportError("ControlService");
 }
 CloseServiceHandle (schService);
 return ret;
}

BOOL OpenDevice(LPCTSTR DriverName) {
 char     completeDeviceName[64];
 LPCTSTR  dosDeviceName = DriverName;
 HANDLE   hDevice;
 BOOL     ret;
    //
    // Create a \\.\XXX device name that CreateFile can use
    //
    // NOTE: We're making an assumption here that the driver
    //       has created a symbolic link using it's own name
    //       (i.e. if the driver has the name "XXX" we assume
    //       that it used IoCreateSymbolicLink to create a
    //       symbolic link "\DosDevices\XXX". Usually, there
    //       is this understanding between related apps/drivers.
    //
    //       An application might also peruse the DEVICEMAP
    //       section of the registry, or use the QueryDosDevice
    //       API to enumerate the existing symbolic links in the
    //       system.
    //

 lstrcpy(completeDeviceName,"\\\\.\\");
 lstrcat(completeDeviceName,dosDeviceName);
 hDevice = CreateFile (completeDeviceName,
		       GENERIC_READ | GENERIC_WRITE,
		       0,NULL,OPEN_EXISTING,FILE_ATTRIBUTE_NORMAL,NULL);
 if (hDevice == ((HANDLE)-1)) {
  printf ("Can't get a handle to %s\n",completeDeviceName);
  ret = FALSE;
 }else{
  printf ("CreateFile SUCCESS\n");
  CloseHandle (hDevice);
  ret = TRUE;
 }
 return ret;
}

VOID __cdecl main(int argc, char *argv[]) {
 SC_HANDLE   schSCManager;

 if (!lstrcmpi(argv[1],"/?")) {
  printf ("usage: instdrv\n");
  printf ("           to install VDONGLE, or:\n");
  printf ("       instdrv remove\n");
  printf ("           to remove VDONGLE\n\n");
  exit (1);
 }

 schSCManager = OpenSCManager (NULL,	// machine (NULL == local)
			       NULL,	// database (NULL == default)
			       SC_MANAGER_ALL_ACCESS); // access required
 if (!schSCManager) {
  printf("SC-Manager tot\n");
  exit(1);
 }
 if (!lstrcmpi(argv[1],"remove")) {
  StopDriver (schSCManager,"vdongle");	//entspricht NET STOP VDONGLE
  RemoveDriver (schSCManager,"vdongle");
 }else{
  InstallDriver (schSCManager,"vdongle","d:/progwin/vdongle/vdongle.sys");
  StartDriver (schSCManager,"vdongle");	//entspricht NET START VDONGLE
  OpenDevice ("VDONGLE.386");		//gucken, ob er da ist
 }
 CloseServiceHandle (schSCManager);
}
